package dev.peerat.backend.routes;

import static dev.peerat.framework.RequestType.POST;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.Map;
import java.util.regex.Matcher;

import org.json.simple.JSONObject;

import dev.peerat.backend.bonus.extract.RouteDoc;
import dev.peerat.backend.model.PeerAtUser;
import dev.peerat.backend.model.Player;
import dev.peerat.backend.repository.DatabaseRepository;
import dev.peerat.framework.Context;
import dev.peerat.framework.HttpReader;
import dev.peerat.framework.HttpWriter;
import dev.peerat.framework.Response;
import dev.peerat.framework.Route;
import dev.peerat.framework.Router;

public class MailConfirmation implements Response {

	private DatabaseRepository databaseRepo;
	private Router<PeerAtUser> router;
	private String usersFilesPath;
	private Map<Player, Integer> playersWaiting;

	public MailConfirmation(DatabaseRepository databaseRepo, Router<PeerAtUser> router, String initUsersFilesPath,
			Map<Player, Integer> playersWaiting) {
		this.databaseRepo = databaseRepo;
		this.router = router;
		usersFilesPath = initUsersFilesPath;
	}

	@RouteDoc(path = "/confirmation", responseCode = 200, responseDescription = "L'utilisateur est inscrit")
	@RouteDoc(responseCode = 403, responseDescription = "L'utilisateur est connecté")
	@RouteDoc(responseCode = 400, responseDescription = "Aucune données fournie / données invalide")

	@Route(path = "^\\/confirmation$", type = POST)
	public void exec(Matcher matcher, Context context, HttpReader reader, HttpWriter writer) throws Exception {
		if (context.getUser() != null) {
			context.response(403);
			return;
		}
		JSONObject informations = reader.readJson();
		if (informations != null) {
			boolean allNecessaryFieldsFilled = informations.containsKey("email") && informations.containsKey("code")
					&& informations.containsKey("passwd");
			if (!allNecessaryFieldsFilled) {
				context.response(400);
				return;
			}
			String email = (String) informations.get("email");
			String password = (String) informations.get("passwd");
			int code = (int) informations.get("code");
	
			Player newPlayer = getPlayerFromEmail(email);
			if (newPlayer != null && code == playersWaiting.get(newPlayer)) {
				String pseudo = newPlayer.getPseudo();
				int id;
				if ((id = databaseRepo.register(pseudo, email, password, newPlayer.getFirstname(), newPlayer.getLastname(), "", "", "")) >= 0) {
					context.response(200, "Access-Control-Expose-Headers: Authorization",
							"Authorization: Bearer " + this.router.createAuthUser(new PeerAtUser(id)));
					createFolderToSaveSourceCode(pseudo);
					return;
				} else {
					context.response(400);
					JSONObject error = new JSONObject();
					error.put("username_valid", pseudo);
					error.put("email_valid", email);
					writer.write(error.toJSONString());
					return;
				}
			}
		}
		context.response(400);
	}

	private void createFolderToSaveSourceCode(String pseudo) throws IOException {

		Files.createDirectories(Paths.get(String.format("%s/%s", usersFilesPath, pseudo)));
	}
	
	private Player getPlayerFromEmail(String email) {
		Player toMatch = new Player(email);
		for (Player p: playersWaiting.keySet()) {
			if (p.equals(toMatch)) {
				return p;
			}
		}
		return null;
	}
}
